//                  in the name of god

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "Config_Glcd_320240.h"
#include "Glcd_GFunctions.h"


// send a word data to the lcd
void lcd_write_index_register(char ins)
    {
    CS_LOW;
    RS_LOW;
    LCD_DATAPORT_MSB_PORT=0x00; 
    LCD_DATAPORT_LSB_PORT=ins;
    WR_LOW; 
    WR_HIGH;
    CS_HIGH;        
    }

// write a word data into the WDR lcd register
void lcd_write_wdr(int data)
    { 
    CS_LOW;
    RS_HIGH;
    LCD_DATAPORT_MSB_PORT=(data>>8); 
    LCD_DATAPORT_LSB_PORT=data; 
    WR_LOW;
    WR_HIGH;
    CS_HIGH;           
    }

// read data from rdr lcd register
int lcd_read_rdr(void)
    { 
    int data_lsb,data_msb;  
    LCD_DATAPORT_CLEAR;
    LCD_DATAPORT_INPUT;
    CS_LOW;
    RS_HIGH;
    WR_HIGH;
    RD_LOW; 
    data_lsb=LCD_DATAPORT_LSB_PIN;
    data_msb=LCD_DATAPORT_MSB_PIN;
    RD_HIGH;
    CS_HIGH;
    return ((data_msb<<8)|data_lsb);
    }

// clears the lcd panel
void lcd_clear_screen(void)
    {  
    unsigned long int counter;
    lcd_write_index_register(TS_INS_GRAM_ADX);
    lcd_write_wdr(0);
    lcd_write_index_register(TS_INS_GRAM_ADY);
    lcd_write_wdr(0);             
    lcd_write_index_register(TS_INS_START_ADX);
    lcd_write_wdr(0);
    lcd_write_index_register(TS_INS_END_ADX);
    #ifdef PORTRAIT
    lcd_write_wdr(239);
    #else
    lcd_write_wdr(319); 
    #endif
    lcd_write_index_register(TS_INS_START_ADY);
    lcd_write_wdr(0);
    lcd_write_index_register(TS_INS_END_ADY);
    #ifdef PORTRAIT
    lcd_write_wdr(319);
    #else
    lcd_write_wdr(239); 
    #endif 
    lcd_write_index_register(TS_INS_RW_GRAM);
    for(counter=0;counter<76800;counter++)
      lcd_write_wdr(0xFFFF); 
    }

// lcd initialization


// turn on a pixel on the lcd screen
void lcd_write_pixel(unsigned int x,unsigned int y,int color)
    { 
    #ifdef LANDSCAPE
    y=239-y;
    #endif
    lcd_write_index_register(TS_INS_GRAM_ADX);
    lcd_write_wdr(x); 
    lcd_write_index_register(TS_INS_GRAM_ADY);
    lcd_write_wdr(y);
    lcd_write_index_register(TS_INS_START_ADX);
    lcd_write_wdr(x);
    lcd_write_index_register(TS_INS_END_ADX);
    lcd_write_wdr(x); 
    lcd_write_index_register(TS_INS_START_ADY);
    lcd_write_wdr(y);
    lcd_write_index_register(TS_INS_END_ADY);
    lcd_write_wdr(y); 
    lcd_write_index_register(TS_INS_RW_GRAM);
    lcd_write_wdr(color);
    }

// draw a line with x0,y0 and x1,y1 coordinates
void lcd_draw_line(int x0,int y0,int x1,int y1,int color) 
    {
    int dy=y1-y0;
    int dx=x1-x0;
    int stepx,stepy;
    if(dy<0) {dy=-dy;stepy=-1;} else {stepy=1;}
    if(dx<0) {dx=-dx;stepx=-1;} else {stepx=1;}
    dy<<=1; 
    dx<<=1; 
    lcd_write_pixel(x0,y0,color);
    if(dx>dy) 
      {
      int fraction=dy-(dx>>1); 
      while(x0!=x1) 
        {
        if(fraction>=0) 
          {
          y0+=stepy;
          fraction-=dx; 
          }
        x0+=stepx;
        fraction+=dy; 
        lcd_write_pixel(x0,y0,color);
        }
      } 
    else 
      {
      int fraction=dx-(dy>>1);
      while (y0 != y1) 
        {
        if(fraction>=0) 
          {
          x0+=stepx;
          fraction-=dy;
          }
        y0+=stepy;
        fraction+=dx;
        lcd_write_pixel(x0,y0,color);
        }
      }
    }

// draw a fill(1) or no fill(0) rectangular with specified color    
void lcd_draw_rectangle(unsigned int x0,unsigned int y0,unsigned int x1,unsigned int y1,char fill,int color)
    { 
    if(fill)
      {
      unsigned int counter_x; 
      unsigned char counter_y; 
      
      lcd_write_index_register(TS_INS_GRAM_ADX);
      lcd_write_wdr(x0);
      lcd_write_index_register(TS_INS_GRAM_ADY);
      lcd_write_wdr(y0);  
      lcd_write_index_register(TS_INS_START_ADX);
      lcd_write_wdr(x0);
      lcd_write_index_register(TS_INS_END_ADX);
      lcd_write_wdr(x1);
      #ifdef PORTRAIT
      lcd_write_index_register(TS_INS_START_ADY);
      lcd_write_wdr(y0);
      lcd_write_index_register(TS_INS_END_ADY);
      lcd_write_wdr(y1); 
      #else
      lcd_write_index_register(TS_INS_START_ADY);
      lcd_write_wdr(239-y1);
      lcd_write_index_register(TS_INS_END_ADY);
      lcd_write_wdr(239-y0);
      #endif  
      lcd_write_index_register(TS_INS_RW_GRAM);
      for(counter_y=y0;counter_y<=y1;counter_y++)
        {
        for(counter_x=x0;counter_x<=x1;counter_x++)
          lcd_write_wdr(color);
        }  
      } 
    else
      {
      lcd_draw_line(x0,y0,x1,y0,color);
      lcd_draw_line(x1,y0,x1,y1,color);
      lcd_draw_line(x1,y1,x0,y1,color); 
      lcd_draw_line(x0,y1,x0,y0,color);
      }
    }

// draw a fill(1) or no fill(0) circle with specified color 
void lcd_draw_circle(unsigned int x0,unsigned int y0,unsigned int radius,char fill,int color) 
    { 
    int x_offset,y_offset,error; 
    x_offset=0; 
    y_offset=radius; 
    error=1-radius; 

    do 
      { 
      if(fill) 
        { 
        lcd_draw_line(x0-x_offset,y0+y_offset,x0+x_offset,y0+y_offset,color); 
        lcd_draw_line(x0-x_offset,y0-y_offset,x0+x_offset,y0-y_offset,color); 
        lcd_draw_line(x0-y_offset,y0+x_offset,x0+y_offset,y0+x_offset,color); 
        lcd_draw_line(x0-y_offset,y0-x_offset,x0+y_offset,y0-x_offset,color); 
        } 
      else 
        { 
        lcd_write_pixel(x_offset+x0,y_offset+y0,color); 
        lcd_write_pixel(y_offset+x0,x_offset+y0,color); 
        lcd_write_pixel(x0-x_offset,y_offset+y0,color); 
        lcd_write_pixel(x0-y_offset,x_offset+y0,color); 
        lcd_write_pixel(y_offset+x0,y0-x_offset,color); 
        lcd_write_pixel(x_offset+x0,y0-y_offset,color); 
        lcd_write_pixel(x0-x_offset,y0-y_offset,color); 
        lcd_write_pixel(x0-y_offset,y0-x_offset,color); 
        } 

      if(error<0) 
        error+=(3+(2*(x_offset++))); 
      else 
        error+=(5+(2*((x_offset++)-(y_offset--)))); 
      }while(x_offset<=y_offset); 
    } 

void lcd_write_pic(unsigned int x,unsigned int y,const prog_uint16_t *  pointer)
     {
	 unsigned int Max_x=pgm_read_word(&pointer[0]);
	 unsigned int Max_y=pgm_read_word(&pointer[1]);

     unsigned long int counter,compare_value=(unsigned long int)Max_x*Max_y;
     lcd_write_index_register(TS_INS_GRAM_ADX);
     lcd_write_wdr(x); 
     lcd_write_index_register(TS_INS_START_ADX);
     lcd_write_wdr(x);
     lcd_write_index_register(TS_INS_END_ADX);
     lcd_write_wdr(x+Max_x-1);   
     #ifdef PORTRAIT
     lcd_write_index_register(TS_INS_GRAM_ADY);
     lcd_write_wdr(y);
     lcd_write_index_register(TS_INS_START_ADY);
     lcd_write_wdr(y);
     lcd_write_index_register(TS_INS_END_ADY);
     lcd_write_wdr(y+Max_y-1);
     #else     
     lcd_write_index_register(TS_INS_GRAM_ADY);
     lcd_write_wdr(239-y);
     lcd_write_index_register(TS_INS_START_ADY);
     lcd_write_wdr(239-(y+Max_y));
     lcd_write_index_register(TS_INS_END_ADY);
     lcd_write_wdr(239-y);
     #endif 
     lcd_write_index_register(TS_INS_RW_GRAM);
     for(counter=0,pointer+=2;counter<compare_value;counter++,pointer++)
       lcd_write_wdr(pgm_read_word(pointer));
     }

void lcd_init(void)
    { 
    LCD_CONTROLPORT_PORT=(1<<LCD_RST_PIN)|(1<<LCD_RS_PIN)|(1<<LCD_CS_PIN)|(1<<LCD_RD_PIN)|(1<<LCD_WR_PIN);
    LCD_CONTROLPORT_DDR=(1<<LCD_RST_PIN)|(1<<LCD_RS_PIN)|(1<<LCD_CS_PIN)|(1<<LCD_RD_PIN)|(1<<LCD_WR_PIN);
    LCD_DATAPORT_OUTPUT;
          
    // reset lcd chipset
    CBI(LCD_RST_PORT,LCD_RST_PIN);
    _delay_ms(1);           
    SBI(LCD_RST_PORT,LCD_RST_PIN);          
    
    lcd_write_index_register(0x00E3);
    lcd_write_wdr(0x3008);                     //set the internal timing
    lcd_write_index_register(0x00E7);
    lcd_write_wdr(0x0012);                     //set the internal timing
    lcd_write_index_register(0x00EF);
    lcd_write_wdr(0x1231);                     //set the internal timing
    lcd_write_index_register(TS_INS_START_OSC);
    lcd_write_wdr(0x0001);                     //start oscillator
    //if(lcd_read_rdr()==0x9325) {DDRB.7=1;PORTB.7=1;}
    _delay_ms(50);

    lcd_write_index_register(TS_INS_DRIV_OUT_CTRL);
    lcd_write_wdr(0x0100);                     //set SS, SM
    lcd_write_index_register(TS_INS_DRIV_WAV_CTRL);
    lcd_write_wdr(0x0700);                     //set 1 line inversion
    
    lcd_write_index_register(TS_INS_ENTRY_MOD);
    lcd_write_wdr(TS_VAL_ENTRY_MOD);            //set GRAM write direction, BGR=0

    lcd_write_index_register(TS_INS_RESIZE_CTRL);
    lcd_write_wdr(0x0000);                     //no resizing

    lcd_write_index_register(TS_INS_DISP_CTRL2);
    lcd_write_wdr(0x0202);                     //front & back porch periods = 2
    lcd_write_index_register(TS_INS_DISP_CTRL3);
    lcd_write_wdr(0x0000);                     
    lcd_write_index_register(TS_INS_DISP_CTRL4);
    lcd_write_wdr(0x0000);                     
    lcd_write_index_register(TS_INS_RGB_DISP_IF_CTRL1);
    lcd_write_wdr(0x0000);                     //select system interface                
    lcd_write_index_register(TS_INS_FRM_MARKER_POS);
    lcd_write_wdr(0x0000);                     
    lcd_write_index_register(TS_INS_RGB_DISP_IF_CTRL2);
    lcd_write_wdr(0x0000);                    
    
    lcd_write_index_register(TS_INS_POW_CTRL1);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_POW_CTRL2);
    lcd_write_wdr(0x0007);                     
    lcd_write_index_register(TS_INS_POW_CTRL3);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_POW_CTRL4);
    lcd_write_wdr(0x0000);                     
    _delay_ms(200);

    lcd_write_index_register(TS_INS_POW_CTRL1);
    lcd_write_wdr(0x1690);
    lcd_write_index_register(TS_INS_POW_CTRL2);
    lcd_write_wdr(0x0007); //lcd_write_wdr(0x0137);                     
    _delay_ms(50);

    lcd_write_index_register(TS_INS_POW_CTRL3);
    lcd_write_wdr(0x001A); //lcd_write_wdr(0x013C);
    _delay_ms(50);

    lcd_write_index_register(TS_INS_POW_CTRL4);
    lcd_write_wdr(0x1800); //lcd_write_wdr(0x1400);
    lcd_write_index_register(TS_INS_POW_CTRL7);
    lcd_write_wdr(0x002A); //lcd_write_wdr(0x0007);
    _delay_ms(50);

    lcd_write_index_register(TS_INS_GRAM_HOR_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GRAM_VER_AD);
    lcd_write_wdr(0x0000);

    lcd_write_index_register(TS_INS_GAMMA_CTRL1);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GAMMA_CTRL2);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GAMMA_CTRL3);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GAMMA_CTRL4);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GAMMA_CTRL5);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_GAMMA_CTRL6);
    lcd_write_wdr(0x0406);
    lcd_write_index_register(TS_INS_GAMMA_CTRL7);
    lcd_write_wdr(0x0006);
    lcd_write_index_register(TS_INS_GAMMA_CTRL8);
    lcd_write_wdr(0x0404);
    lcd_write_index_register(TS_INS_GAMMA_CTRL9);
    lcd_write_wdr(0x0700);
    lcd_write_index_register(TS_INS_GAMMA_CTRL10);
    lcd_write_wdr(0x0A08);

    lcd_write_index_register(TS_INS_HOR_START_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_HOR_END_AD);
    lcd_write_wdr(0x00EF);
    lcd_write_index_register(TS_INS_VER_START_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_VER_END_AD);
    lcd_write_wdr(0x013F);
    lcd_write_index_register(TS_INS_GATE_SCAN_CTRL1);
    lcd_write_wdr(0xA700);
    lcd_write_index_register(TS_INS_GATE_SCAN_CTRL2);
    lcd_write_wdr(0x0001);
    lcd_write_index_register(TS_INS_GATE_SCAN_CTRL3);
    lcd_write_wdr(0x0000);

    lcd_write_index_register(TS_INS_PART_IMG1_DISP_POS);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PART_IMG1_START_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PART_IMG1_END_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PART_IMG2_DISP_POS);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PART_IMG2_START_AD);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PART_IMG2_END_AD);
    lcd_write_wdr(0x0000);

    lcd_write_index_register(TS_INS_PANEL_IF_CTRL1);
    lcd_write_wdr(0x0010);
    lcd_write_index_register(TS_INS_PANEL_IF_CTRL2);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PANEL_IF_CTRL3);
    lcd_write_wdr(0x0003);
    lcd_write_index_register(TS_INS_PANEL_IF_CTRL4);
    lcd_write_wdr(0x0110);
    lcd_write_index_register(TS_INS_PANEL_IF_CTRL5);
    lcd_write_wdr(0x0000);
    lcd_write_index_register(TS_INS_PANEL_IF_CTRL6);
    lcd_write_wdr(0x0000);

    lcd_write_index_register(TS_INS_DISP_CTRL1);
    lcd_write_wdr(0x0133);
    
    lcd_write_index_register(TS_INS_FRM_RATE_COL_CTRL);
    lcd_write_wdr(0x000D);
    
    lcd_clear_screen();
    }
  
